@@ -262,7 +262,7 @@ static inline void
 handle_atk_action(GladeParseState *state, const xmlChar **attrs)
 {
     gint i;
-    GladeAtkActionInfo info = { NULL };
+    GladeAtkActionInfo info = { 0 };
 
     flush_properties(state);
 
@@ -270,8 +270,7 @@ handle_atk_action(GladeParseState *state, const xmlChar **attrs)
 	if (!strcmp(attrs[i], "action_name"))
 	    info.action_name = alloc_string(state->interface, attrs[i+1]);
 	else if (!strcmp(attrs[i], "description"))
-	    info.description = alloc_string(state->interface,
-				    dgettext(state->domain, attrs[i+1]));
+	    info.description = alloc_string(state->interface, attrs[i+1]);
 	else
 	    g_warning("unknown attribute `%s' for <action>.", attrs[i]);
     }
@@ -289,7 +288,7 @@ static inline void
 handle_atk_relation(GladeParseState *state, const xmlChar **attrs)
 {
     gint i;
-    GladeAtkRelationInfo info = { NULL };
+    GladeAtkRelationInfo info = { 0 };
 
     flush_properties(state);
 
@@ -314,7 +313,7 @@ handle_atk_relation(GladeParseState *state, const xmlChar **attrs)
 static inline void
 handle_signal(GladeParseState *state, const xmlChar **attrs)
 {
-    GladeSignalInfo info = { NULL };
+    GladeSignalInfo info = { 0 };
     gint i;
 
     flush_properties(state);
@@ -592,8 +591,6 @@ glade_parser_start_element(GladeParseState *state,
 		    state->context_prop = !strcmp(attrs[i+1], "yes");
 		else if (!strcmp(attrs[i], "agent"))
 		    bad_agent = strcmp(attrs[i], "libglade") != 0;
-		else if (!strcmp(attrs[i], "comments"))
-		    /* Do nothing. */;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -650,8 +647,6 @@ glade_parser_start_element(GladeParseState *state,
 		    state->translate_prop = !strcmp(attrs[i+1], "yes");
 		else if (!strcmp(attrs[i], "context"))
 		    state->context_prop = !strcmp(attrs[i+1], "yes");
-		else if (!strcmp(attrs[i], "comments"))
-		    /* Do nothing. */;
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
 			      attrs[i]);
@@ -805,8 +800,6 @@ glade_parser_start_element(GladeParseState *state,
 		    state->context_prop = !strcmp(attrs[i+1], "yes");
 		else if (!strcmp(attrs[i], "agent"))
 		    bad_agent = strcmp(attrs[i], "libglade") != 0;
-		else if (!strcmp(attrs[i], "comments"))
-		    /* Do nothing. */;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -1100,30 +1093,30 @@ glade_parser_fatal_error(GladeParseState *state, const char *msg, ...)
 }
 
 static xmlSAXHandler glade_parser = {
-    (internalSubsetSAXFunc)NULL,                       /* internalSubset */
-    (isStandaloneSAXFunc)NULL,                         /* isStandalone */
-    (hasInternalSubsetSAXFunc)NULL,                    /* hasInternalSubset */
-    (hasExternalSubsetSAXFunc)NULL,                    /* hasExternalSubset */
-    (resolveEntitySAXFunc)NULL,                        /* resolveEntity */
-    (getEntitySAXFunc)glade_parser_get_entity,         /* getEntity */
-    (entityDeclSAXFunc)NULL,                           /* entityDecl */
-    (notationDeclSAXFunc)NULL,                         /* notationDecl */
-    (attributeDeclSAXFunc)NULL,                        /* attributeDecl */
-    (elementDeclSAXFunc)NULL,                          /* elementDecl */
-    (unparsedEntityDeclSAXFunc)NULL,                   /* unparsedEntityDecl */
-    (setDocumentLocatorSAXFunc)NULL,                   /* setDocumentLocator */
+    0, /* internalSubset */
+    0, /* isStandalone */
+    0, /* hasInternalSubset */
+    0, /* hasExternalSubset */
+    0, /* resolveEntity */
+    (getEntitySAXFunc)glade_parser_get_entity, /* getEntity */
+    0, /* entityDecl */
+    0, /* notationDecl */
+    0, /* attributeDecl */
+    0, /* elementDecl */
+    0, /* unparsedEntityDecl */
+    0, /* setDocumentLocator */
     (startDocumentSAXFunc)glade_parser_start_document, /* startDocument */
-    (endDocumentSAXFunc)glade_parser_end_document,     /* endDocument */
-    (startElementSAXFunc)glade_parser_start_element,   /* startElement */
-    (endElementSAXFunc)glade_parser_end_element,       /* endElement */
-    (referenceSAXFunc)NULL,                            /* reference */
-    (charactersSAXFunc)glade_parser_characters,        /* characters */
-    (ignorableWhitespaceSAXFunc)NULL,               /* ignorableWhitespace */
-    (processingInstructionSAXFunc)NULL,             /* processingInstruction */
-    (commentSAXFunc)NULL,                              /* comment */
-    (warningSAXFunc)glade_parser_warning,              /* warning */
-    (errorSAXFunc)glade_parser_error,                  /* error */
-    (fatalErrorSAXFunc)glade_parser_fatal_error,       /* fatalError */
+    (endDocumentSAXFunc)glade_parser_end_document, /* endDocument */
+    (startElementSAXFunc)glade_parser_start_element, /* startElement */
+    (endElementSAXFunc)glade_parser_end_element, /* endElement */
+    0, /* reference */
+    (charactersSAXFunc)glade_parser_characters, /* characters */
+    0, /* ignorableWhitespace */
+    0, /* processingInstruction */
+    (commentSAXFunc)0, /* comment */
+    (warningSAXFunc)glade_parser_warning, /* warning */
+    (errorSAXFunc)glade_parser_error, /* error */
+    (fatalErrorSAXFunc)glade_parser_fatal_error, /* fatalError */
 };
 
 static void
@@ -1136,8 +1129,6 @@ widget_info_free(GladeWidgetInfo *info)
     g_free(info->properties);
     g_free(info->atk_props);
     g_free(info->signals);
-    g_free(info->atk_actions);
-    g_free(info->relations);
     g_free(info->accels);
 
     for (i = 0; i < info->n_children; i++) {
